/*
 *      Program do przegladania logow z ekg.
 *      Linia komend.
 *
 *
 *      copyright (c) 2003 by Lukasz Bartnik
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License Version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "cmdline.h"

#include <cstring>

cmdline_t::cmdline_t(unsigned int width, WINDOW * win)
    :_line_pos(0),_history_size(100),_history_used(false),_kp(1,1)
{
	keypad(_win=win, TRUE);
	_width = width;
}


const std::string cmdline_t::get_cmd()
{
	// Przydatne do porownywania.
	static key_pair enter(0, 0x0A), up(0, KEY_UP), down(0, KEY_DOWN),
			left(0, KEY_LEFT), right(0, KEY_RIGHT), backspc(0, KEY_BACKSPACE),
			pgup(0, KEY_PPAGE), pgdown(0, KEY_NPAGE), del(0, KEY_DC),
			home(0, KEY_HOME), end(0, KEY_END);

	std::map<key_pair, std::string>::iterator i;


	while(true) {
		_kp.add(wgetch(_win));

		/* Jesli linia komend jest aktywna, to przelapuje sporo
		 * klawiszy kontrolnych. */
		if (_kp == enter) {
			// czy bedziemy dodawac linie do historii
			bool into_history = true;

			if (!_active)
				return "key_enter";

			// jesli wcisnieto sam enter
			if (_line.size() == 0) {
				if (_history.size() == 0)
					continue;

				_line = _history.front();
				into_history = false;
			}

			_argv.clear();
			try {
				parse(_line);
			}
			// gdyby byl wyjatek, to trzeba wyczyscic linie
			catch(cmdline_error_t &) {
				_line.clear();
				_line_pos = 0;
				throw;
			}

			// dodajmy komende do historii
			if (into_history) {
				if (_history.size() >= _history_size)
					_history.pop_back();

				_history.push_front(_line);
			}

			// kosmetyka linii polecen
			_line.clear();
			_line_pos = 0;
			_history_iter = _history.begin();
			_history_used = false;

			return _command;

		} else if (_kp == up && _active) {
			if (!_history.size())
				continue;

			if (_history_used) {
				++_history_iter;
				if (_history_iter == _history.end())
					--_history_iter;

				_line = *_history_iter;
				_line_pos = _line.size();
			} else {
				_line = *_history_iter;
				_line_pos = _line.size();
				_history_used = true;
			}

		} else if (_kp == down && _active) {
			if (!_history.size())
				continue;

			if (_history_used) {
				if (_history_iter == _history.begin()) {
					_line.erase();
					_line_pos = 0;
					_history_used = false;
				} else {
					--_history_iter;
					_line = *_history_iter;
					_line_pos = _line.size();
				}
			}

		} else if (_kp == left && _active) {
			if (_line_pos > 0)
				--_line_pos;

		} else if (_kp == right && _active) {
			if (_line_pos < _line.size())
				++_line_pos;

		} else if (_kp == backspc) {
			if (_line.size() && _line_pos) {
				_line.erase(_line_pos-1,1);
				--_line_pos;
			}

		} else if (_kp == del && _active) {
			if (_line.size() && _line_pos < _line.size())
				_line.erase(_line_pos,1);

		} else if (_kp == home && _active) {
			_line_pos = 0;

		} else if (_kp == end && _active) {
			_line_pos = _line.size();

		/* Jesli nie da sie dopasowac, to moze mozna to zwrocic na
		 * zewnatrz... */
		} else if ((i = _spc_keys.find(_kp)) != _spc_keys.end()) {
			return i->second;

		/* A jesli nie, to trzeba dodac do tresci */
		} else {
			if(!isprint(_kp.k2))
				continue;

			if(_line_pos == _line.size()) 
				_line += (char)_kp.k2;
			else
				_line.insert(_line_pos,1,(char)_kp.k2);

			++_line_pos;
		}

		/* po kazdym przebiegu petli trzeba wypisac tresc linii komend */
		print();
		doupdate();
	}
}


void cmdline_t::set_active(bool a)
{
	_active = a;
}


void cmdline_t::print()
{
	werase(_win);

	unsigned int offset = 0;

	// przesuwamy widoczna czesc linii tak, by pozostaly trzy ostatnie znaki
	while(_line_pos-offset >= _width)
		offset += _width-3;

	mvwprintw(_win, 0, 0, const_cast<char*> (_line.substr(offset, std::string::npos).c_str()));
	wmove(_win, 0, _line_pos-offset);

	wnoutrefresh(_win);
}


void cmdline_t::register_key(int key, const std::string & descr)
{
	register_sequence(0, key, descr);
}

void cmdline_t::register_sequence(int key1, int key2, const std::string & descr)
{
	_spc_keys[key_pair(key1, key2)]          = descr;
}


const std::vector<std::string> & cmdline_t::argv()
{
	return _argv;
}


void cmdline_t::parse(const std::string & str)
{
	unsigned int i=0;
	while(isspace(str[i]))
	    ++i;

	_command = parse_get_piece(str, i);

	while(i < str.size()) {
		while(isspace(str[i]))
			++i;

		if (i<str.size())
			_argv.push_back(parse_get_piece(str, i));
	}
}


std::string cmdline_t::parse_get_piece(const std::string & str, unsigned int & i)
{
	std::string ret;

	if (str[i] == '\"') {
		++i;
		/* przeszukujemy ciag az do nastepnego znaku "
		 * pomijamy wszystkie znaki po \ */
		while (str[i] != '\"' && i<str.size()) {
			if (str[i] == '\\') {
				/* po znaku \ cos musi jeszcze stac */
				if (++i >= str.size())
					throw cmdline_error_t(PARSE_ERROR, str[i-1]);

				switch(str[i]) {
					/* tylko te znaki moga wystepowac po znaku \ */
					case '\'':
					case '\"':
					case '\\':
						ret += str[i];
						break;
					default:
						throw cmdline_error_t(PARSE_ERROR, str[i]);
						break;
				}
			} else {
				ret += str[i];
			}

			++i;
		}
		if (str[i] != '\"') {
			throw cmdline_error_t(WRONG_END, 0);
		}

		// przesunac iterator na nastepny znak do sprawdzania
		++i;
		return ret;
	} else {
		// jesli nie ma na poczatku "
		while (str[i] != ' ' && i<str.size()) {
			if (str[i] == '\\') {
				/* po znaku \ cos musi jeszcze stac */
				if (++i >= str.size())
					throw cmdline_error_t(PARSE_ERROR, str[i-1]);

				switch(str[i]) {
					/* tylko te znaki moga wystepowac po znaku \ */
					case '\'':
					case '\"':
					case '\\':
					case ' ':
						ret += str[i];
						break;
					default:
						throw cmdline_error_t(PARSE_ERROR, str[i]);
						break;
				}
			} else {
				ret += str[i];
			}

			++i;
		}

		// przesunac iterator na nastepny znak do sprawdzania
		++i;
		return ret;
	}
}


bool cmdline_t::is_good_to_complete()
{
	// wolno dopelniac tylko na koncu linii; ostatnia nie moze byc spacja
	if (_line_pos != _line.size())// || _line[_line.size()-1] == ' ')
		return false;

	return true;
}


std::string cmdline_t::get_last()
{
	unsigned int i=0, start=0;
	std::string arg;

	while(i < _line.size()) {
		while(isspace(_line[i]))
			++i;

		if (i<_line.size()) {
			start = i;
			try {
				if (i<_line.size())
					arg = parse_get_piece(_line, i);
			}
			catch (cmdline_error_t &e) {
				if (e.type == WRONG_END && i == _line.size())
					return _line.substr(start+1, std::string::npos);
			}
		}
	}

	return arg;
}


std::string cmdline_t::get_first()
{
	unsigned int i=0;
	std::string arg;

	try {
		if (i<_line.size())
			arg = parse_get_piece(_line, i);
	}
	catch (cmdline_error_t &e) {
		if (e.type == WRONG_END)
			return _line.substr(1, std::string::npos);
	}

	return arg;
}


bool cmdline_t::one_piece()
{
	unsigned int i=0;

	try {
		if (i<_line.size())
			parse_get_piece(_line, i);
	}
	catch (cmdline_error_t &e) {
	}

	// pomijamy jeszcze biale znaki
	while(isspace(_line[i]))
		++i;

	// i jest wieksze lub rowne _line.size() tylko wtedy, gdy w linii
	// jest tylko jeden ciagly wpis - dopelniamy komende
	if (i >= _line.size() || _line.size()==0)
		return true;

	return false;
}

bool cmdline_t::is_quoted() const
{
	bool q = false;

	for (std::string::const_iterator i=_line.begin(); i!=_line.end(); ++i)
		switch (*i) {
			case '\\':
				++i;
				break;
			case '\"':
				q = !q;
				break;
		}

	return q;
}

void cmdline_t::add_part_completition(const std::string & str)
{
	_line += str;
	_line_pos = _line.size();
}


void cmdline_t::add_completition(const std::string & str)
{
	_line += str;
	if (is_quoted())
		_line += "\" ";
	else
		_line += " ";

	_line_pos = _line.size();
}
	
