/*
 *      Program do przegladania logow z ekg.
 *      Wszelke komendy (klawisze i ich kombinacje) sterujace
 *      interfejsem uzytkownika.
 *
 *
 *      copyright (c) 2003 by Lukasz Bartnik
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License Version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __UI_COMMANDS_HH__
#define __UI_COMMANDS_HH__

#include "texts.h"

#include "command-base.h"

/**
 * @brief Baza dla komend interfejsu.
 *
 * Wszelkie komendy interfejsu powinny dziedziczyc z tej klasy.
 * Ma ona zaimplementowane funkcje dopelniania, ktore dla komend
 * interfejsu nie powinny nic zwracac.
 */
class command_ui_base_t : public command_base_t {
public:

	command_ui_base_t(const std::string & name) : command_base_t(name) {}

	/// Pierwszy poziom dopelniania - czyli nazwa komendy.
	virtual const std::string & completition_lvl1()
	{
		return _null;
	}

	/// drugi poziom dopelniania: mozliwe argumenty komendy
	virtual const std::vector<std::string> & completition_lvl2()
	{
		return _null_vector;
	}

	/// klawisz nie maja zandej pomocy
	virtual const std::string & help_short()
	{
		return _null;
	}

	/// klawisz nie maja zandej pomocy
	virtual const std::string & help_long()
	{
		return _null;
	}


protected:

	void set_all_passive()
	{
		_info->set_passive();
		_main->set_passive();
		_nicks->set_passive();
		_chats->set_passive();
		_cmdline->set_active(false);
	}

	/// pusty string - dla komend, ktore nic nie zwracaja
	static std::string _null;

	/// pusty vector stringow - rowniez dla dopelniania
	static std::vector<std::string> _null_vector;
};


/**
 * @brief Klawisz f1 - czyli krotkie info.
 */
class uicmd_f1_t : public command_ui_base_t {
public:

	uicmd_f1_t() : command_ui_base_t("key_f1") {}

	void run(const std::vector<std::string> & args)
	{
		_info->print(std::string(INFO_F1), std::string(INFO_F1_FMT));
	}
};


/**
 *@brief F3 - chowanie bocznych list.
 */
class uicmd_f3_t : public command_ui_base_t {
public:

	uicmd_f3_t() : command_ui_base_t("key_f3"),_shown(true) {}

	void run(const std::vector<std::string> & args)
	{
		_shown = !_shown;
		_nicks->set_displayed(_shown);
		_chats->set_displayed(_shown);
	}

private:
	bool _shown;
};



/**
 * @brief Kombinacja alt+1 - przelacza na okno glowne.
 */
class uicmd_alt1_t : public command_ui_base_t {
public:

	uicmd_alt1_t() : command_ui_base_t("alt_1") {}

	void run(const std::vector<std::string> & args)
	{
		set_all_passive();
		_info->set_displayed(false);
		_main->set_displayed(true);
		_main->set_active();
		_status->set_chosen(STATUS_CHOSEN_MAIN);
		_active_window = _main;
	}
};


/**
 * @brief Kombinacja alt+2 - przelacza na okno glowne (informacje).
 */
class uicmd_alt2_t : public command_ui_base_t {
public:

	uicmd_alt2_t() : command_ui_base_t("alt_2") {}

	void run(const std::vector<std::string> & args)
	{
		set_all_passive();
		_info->set_displayed(true);
		_main->set_displayed(false);
		_info->set_active();
		_status->set_chosen(STATUS_CHOSEN_INFO);
		_active_window = _info;
	}
};


/**
 * @brief Kombinacja alt+3 - przelacza na liste nikow.
 */
class uicmd_alt3_t : public command_ui_base_t {
public:

	uicmd_alt3_t() : command_ui_base_t("alt_3") {}

	void run(const std::vector<std::string> & args)
	{
		set_all_passive();
		_nicks->set_active();
		_status->set_chosen(STATUS_CHOSEN_NICKS);
		_active_window = _nicks;
	}
};


/**
 * @brief Kombinacja alt+4 - przelacza na liste rozmow.
 */
class uicmd_alt4_t : public command_ui_base_t {
public:

	uicmd_alt4_t() : command_ui_base_t("alt_4") {}

	void run(const std::vector<std::string> & args)
	{
		set_all_passive();
		_chats->set_active();
		_status->set_chosen(STATUS_CHOSEN_CHATS);
		_active_window = _chats;
	}
};


/**
 * @brief Kombinacja alt+5 - przelacza na linie komend.
 */
class uicmd_alt5_t : public command_ui_base_t {
public:

	uicmd_alt5_t() : command_ui_base_t("alt_5") {}

	void run(const std::vector<std::string> & args)
	{
		set_all_passive();
		_cmdline->set_active(true);
		_status->set_chosen(STATUS_CHOSEN_CMDLINE, winstatus_t::ACTIVE);
		_active_window = 0;
	}
};


/**
 * @brief Strzalka w gore.
 */
class uicmd_up_t : public command_ui_base_t {
public:

	uicmd_up_t() : command_ui_base_t("key_up") {}

	void run(const std::vector<std::string> & args)
	{
		if(!_active_window)
			return;

		_active_window->moveup();
		if(_active_window == _nicks)
			reload_chats();

	}
};


/**
 * @brief Strzalka w dol.
 */
class uicmd_down_t : public command_ui_base_t {
public:

	uicmd_down_t() : command_ui_base_t("key_down") {}

	void run(const std::vector<std::string> & args)
	{
		if(!_active_window)
			return;

		_active_window->movedown();
		if(_active_window == _nicks)
			reload_chats();
	}
};


/**
 * @brief Poprzednia strona.
 */
class uicmd_pgup_t : public command_ui_base_t {
public:

	uicmd_pgup_t() : command_ui_base_t("key_pgup") {}

	void run(const std::vector<std::string> & args)
	{
		if(!_active_window)
			return;

		_active_window->movepgup();
		if(_active_window == _nicks)
			reload_chats();
	}
};


/**
 * @brief Nastepna strona.
 */
class uicmd_pgdown_t : public command_ui_base_t {
public:

	uicmd_pgdown_t() : command_ui_base_t("key_pgdown") {}

	void run(const std::vector<std::string> & args)
	{
		if(!_active_window)
			return;

		_active_window->movepgdown();
		if(_active_window == _nicks)
			reload_chats();
	}
};



/**
 * @brief Potwierdzenie wyboru.
 */
class uicmd_enter_t : public command_ui_base_t {
public:

	uicmd_enter_t() : command_ui_base_t("key_enter") {}

	void run(const std::vector<std::string> & args)
	{
		if (_active_window == _nicks) {
			_nicks->set_passive();
			_chats->set_active();
			_status->set_chosen(STATUS_CHOSEN_CHATS);
			_active_window = _chats;
		} else if (_active_window == _chats) {
			UinLog & log = _db->log(_nicks->get_chosen_value())->second;
			int t = _chats->get_chosen_value();
			if (t)
				_main->add_chat(log.chat(t)->second, log);
			else
				_main->add_statuses(log);

			_info->set_displayed(false);
			_main->set_displayed(true);
			_main->set_passive();
		}
	}
};


/**
 * @brief Obsluga klawisza Home.
 */
class uicmd_home_t : public command_ui_base_t {
public:
	uicmd_home_t() : command_ui_base_t("key_home") {}

	void run(const std::vector<std::string> & )
	{
		if (_active_window)
			_active_window->movehome();
	}
};


/**
 * @brief Obsluga klawisza End.
 */
class uicmd_end_t : public command_ui_base_t {
public:
	uicmd_end_t() : command_ui_base_t("key_end") {}

	void run(const std::vector<std::string> & )
	{
		if (_active_window)
			_active_window->moveend();
	}
};



#endif /* __UI_COMMANDS_HH__ */
